@@ -145,10 +145,21 @@ impl<T> RunState for &'_ mut T where T: RunState {
 	}
 
 	fn finish(&mut self, writer: Self::Writer) {
-		(*self).finish(writer)
+		(*self).finish(writer);
 	}
 }
 
+/// Whether [`run`] should generate an impl of [`schemars::JsonSchema`] for the type or not.
+#[derive(Clone, Copy, Debug)]
+pub enum GenerateSchema<'a> {
+	Yes {
+		/// An optional feature that the impl of [`schemars::JsonSchema`] will be `cfg`-gated by.
+		feature: Option<&'a str>,
+	},
+
+	No,
+}
+
 /// Each invocation of this function generates a single type specified by the `definition_path` parameter along with its associated API operation functions.
 ///
 /// # Parameters
@@ -164,7 +175,7 @@ impl<T> RunState for &'_ mut T where T: RunState {
 ///
 /// - `vis`: The visibility modifier that should be emitted on the generated code.
 ///
-/// - `optional_feature`: If specified, all API functions will be emitted with a `#[cfg(feature = "<this value>")]` attribute.
+/// - `operation_feature`: If specified, all API functions will be emitted with a `#[cfg(feature = "<this value>")]` attribute.
 ///    The attribute will also be applied to their optional parameters and response types, if any, and to common types for
 ///    optional parameters and response types that are shared by multiple operations.
 ///
@@ -175,7 +186,8 @@ pub fn run(
 	definition_path: &swagger20::DefinitionPath,
 	map_namespace: &impl MapNamespace,
 	vis: &str,
-	optional_feature: Option<&str>,
+	generate_schema: GenerateSchema<'_>,
+	operation_feature: Option<&str>,
 	mut state: impl RunState,
 ) -> Result<RunResult, Error> {
 	use std::io::Write;
@@ -196,22 +208,26 @@ pub fn run(
 		.into_iter()
 		.collect();
 
-	let type_feature = match &definition.kind {
-		swagger20::SchemaKind::Ty(swagger20::Type::CreateOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::DeleteOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ListOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::PatchOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ReplaceOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::WatchOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::CreateResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::DeleteResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ListResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::PatchResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ReplaceResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::WatchResponse) => optional_feature,
-
-		_ => None,
-	};
+	let type_feature =
+		if let swagger20::SchemaKind::Ty(
+			swagger20::Type::CreateOptional(_) |
+			swagger20::Type::DeleteOptional(_) |
+			swagger20::Type::ListOptional(_) |
+			swagger20::Type::PatchOptional(_) |
+			swagger20::Type::ReplaceOptional(_) |
+			swagger20::Type::WatchOptional(_) |
+			swagger20::Type::CreateResponse |
+			swagger20::Type::DeleteResponse |
+			swagger20::Type::ListResponse |
+			swagger20::Type::PatchResponse |
+			swagger20::Type::ReplaceResponse |
+			swagger20::Type::WatchResponse
+		) = &definition.kind {
+			operation_feature
+		}
+		else {
+			None
+		};
 
 	let mut out = state.make_writer(&namespace_parts, type_feature)?;
 
@@ -255,15 +271,13 @@ pub fn run(
 						}
 					}
 
-					let field_name = get_rust_ident(&name);
+					let field_name = get_rust_ident(name);
 
 					let mut field_type_name = String::new();
 
-					let required = match (required, &schema.kind) {
-						(true, _) => templates::PropertyRequired::Required,
-						(false, swagger20::SchemaKind::Ty(swagger20::Type::Array { .. })) |
-						(false, swagger20::SchemaKind::Ty(swagger20::Type::Object { .. })) => templates::PropertyRequired::OptionalDefault,
-						(false, _) => templates::PropertyRequired::Optional,
+					let required = match required {
+						true => templates::PropertyRequired::Required,
+						false => templates::PropertyRequired::Optional,
 					};
 
 					if let templates::PropertyRequired::Optional = required {
@@ -291,7 +305,7 @@ pub fn run(
 							) => {
 								field_type_name.push_str("Box<");
 								field_type_name.push_str(&type_name);
-								field_type_name.push_str(">");
+								field_type_name.push('>');
 							},
 
 							_ => field_type_name.push_str(&type_name),
@@ -302,7 +316,7 @@ pub fn run(
 					};
 
 					if let templates::PropertyRequired::Optional = required {
-						field_type_name.push_str(">");
+						field_type_name.push('>');
 					}
 
 					let is_flattened = matches!(&schema.kind, swagger20::SchemaKind::Ty(swagger20::Type::CustomResourceSubresources(_)));
@@ -361,7 +375,7 @@ pub fn run(
 				kubernetes_group_kind_versions.sort();
 
 				let mut operations_by_gkv: std::collections::BTreeMap<_, Vec<_>> = Default::default();
-				for operation in std::mem::replace(operations, vec![]) {
+				for operation in std::mem::take(operations) {
 					operations_by_gkv
 						.entry(operation.kubernetes_group_kind_version.clone())
 						.or_default()
@@ -384,16 +398,18 @@ pub fn run(
 									map_namespace,
 									vis,
 									Some(type_name),
-									optional_feature)?;
+									operation_feature)?;
 							state.handle_operation_types(operation_optional_parameters_name.as_deref(), operation_result_name.as_deref())?;
 							run_result.num_generated_apis += 1;
 
 							// If this is a CRUD operation, use it to determine the resource's URL path segment and scope.
 							match operation.kubernetes_action {
-								Some(swagger20::KubernetesAction::Delete) |
-								Some(swagger20::KubernetesAction::Get) |
-								Some(swagger20::KubernetesAction::Post) |
-								Some(swagger20::KubernetesAction::Put) => (),
+								Some(
+									swagger20::KubernetesAction::Delete |
+									swagger20::KubernetesAction::Get |
+									swagger20::KubernetesAction::Post |
+									swagger20::KubernetesAction::Put
+								) => (),
 								_ => continue,
 							}
 							let mut components = operation.path.rsplit('/');
@@ -404,6 +420,10 @@ pub fn run(
 								components.next(),
 							);
 
+							// TODO:
+							// The clippy lint doesn't take into account that the order of the arms is significant.
+							// Ref: https://github.com/rust-lang/rust-clippy/issues/860
+							#[allow(clippy::match_same_arms)]
 							let (url_path_segment_, scope_, url_path_segment_and_scope) = match components {
 								("{name}", Some(url_path_segment), Some("{namespace}"), Some("namespaces")) =>
 									(format!("{:?}", url_path_segment), format!("{}NamespaceResourceScope", local), &mut namespace_or_cluster_scoped_url_path_segment_and_scope),
@@ -430,7 +450,7 @@ pub fn run(
 					}
 				}
 
-				*operations = operations_by_gkv.into_iter().flat_map(|(_, operations)| operations).collect();
+				*operations = operations_by_gkv.into_values().flatten().collect();
 			}
 
 			match &**definition_path {
@@ -459,7 +479,7 @@ pub fn run(
 					list_kind: list_kind.as_deref(),
 					metadata_ty: Some(metadata_ty),
 					url_path_segment_and_scope: match (&*namespace_or_cluster_scoped_url_path_segment_and_scope, &*subresource_url_path_segment_and_scope) {
-						([(url_path_segment, scope)], _) => (&**url_path_segment, &**scope),
+						([(url_path_segment, scope)], _) |
 						([], [(url_path_segment, scope)]) => (&**url_path_segment, &**scope),
 
 						([], []) => return Err(format!(
@@ -474,8 +494,7 @@ pub fn run(
 					},
 				}),
 
-				(Some(_), Some((_, templates::PropertyRequired::Optional))) |
-				(Some(_), Some((_, templates::PropertyRequired::OptionalDefault))) =>
+				(Some(_), Some((_, templates::PropertyRequired::Optional | templates::PropertyRequired::OptionalDefault))) =>
 					return Err(format!("definition {} has optional metadata", definition_path).into()),
 
 				(
@@ -489,7 +508,7 @@ pub fn run(
 					list_kind: list_kind.as_deref(),
 					metadata_ty: None,
 					url_path_segment_and_scope: match (&*namespace_or_cluster_scoped_url_path_segment_and_scope, &*subresource_url_path_segment_and_scope) {
-						([(url_path_segment, scope)], _) => (&**url_path_segment, &**scope),
+						([(url_path_segment, scope)], _) |
 						([], [(url_path_segment, scope)]) => (&**url_path_segment, &**scope),
 
 						([], []) => return Err(format!(
@@ -566,9 +585,11 @@ pub fn run(
 			run_result.num_generated_structs += 1;
 		},
 
-		swagger20::SchemaKind::Ty(ty @ swagger20::Type::JsonSchemaPropsOrArray(_)) |
-		swagger20::SchemaKind::Ty(ty @ swagger20::Type::JsonSchemaPropsOrBool(_)) |
-		swagger20::SchemaKind::Ty(ty @ swagger20::Type::JsonSchemaPropsOrStringArray(_)) => {
+		swagger20::SchemaKind::Ty(ty @ (
+			swagger20::Type::JsonSchemaPropsOrArray(_) |
+			swagger20::Type::JsonSchemaPropsOrBool(_) |
+			swagger20::Type::JsonSchemaPropsOrStringArray(_)
+		)) => {
 			let (namespace, json_schema_props_or) = match ty {
 				swagger20::Type::JsonSchemaPropsOrArray(namespace) => (namespace, templates::json_schema_props_or::Or::Array),
 				swagger20::Type::JsonSchemaPropsOrBool(namespace) => (namespace, templates::json_schema_props_or::Or::Bool),
@@ -766,12 +787,14 @@ pub fn run(
 			run_result.num_generated_structs += 1;
 		},
 
-		swagger20::SchemaKind::Ty(ty @ swagger20::Type::CreateOptional(_)) |
-		swagger20::SchemaKind::Ty(ty @ swagger20::Type::DeleteOptional(_)) |
-		swagger20::SchemaKind::Ty(ty @ swagger20::Type::ListOptional(_)) |
-		swagger20::SchemaKind::Ty(ty @ swagger20::Type::PatchOptional(_)) |
-		swagger20::SchemaKind::Ty(ty @ swagger20::Type::ReplaceOptional(_)) |
-		swagger20::SchemaKind::Ty(ty @ swagger20::Type::WatchOptional(_)) => {
+		swagger20::SchemaKind::Ty(ty @ (
+			swagger20::Type::CreateOptional(_) |
+			swagger20::Type::DeleteOptional(_) |
+			swagger20::Type::ListOptional(_) |
+			swagger20::Type::PatchOptional(_) |
+			swagger20::Type::ReplaceOptional(_) |
+			swagger20::Type::WatchOptional(_)
+		)) => {
 			let properties = match ty {
 				swagger20::Type::CreateOptional(properties) |
 				swagger20::Type::DeleteOptional(properties) |
@@ -836,7 +859,7 @@ pub fn run(
 						vis,
 						&template_properties,
 						false,
-						optional_feature,
+						operation_feature,
 						map_namespace,
 					)?,
 
@@ -858,7 +881,7 @@ pub fn run(
 						vis,
 						&template_properties,
 						true,
-						optional_feature,
+						operation_feature,
 						map_namespace,
 					)?,
 
@@ -874,7 +897,7 @@ pub fn run(
 				type_name,
 				map_namespace,
 				templates::operation_response_common::OperationAction::Create,
-				optional_feature,
+				operation_feature,
 			)?;
 
 			run_result.num_generated_structs += 1;
@@ -886,7 +909,7 @@ pub fn run(
 				type_name,
 				map_namespace,
 				templates::operation_response_common::OperationAction::Delete,
-				optional_feature,
+				operation_feature,
 			)?;
 
 			run_result.num_generated_structs += 1;
@@ -898,7 +921,7 @@ pub fn run(
 				type_name,
 				map_namespace,
 				templates::operation_response_common::OperationAction::List,
-				optional_feature,
+				operation_feature,
 			)?;
 
 			run_result.num_generated_structs += 1;
@@ -910,7 +933,7 @@ pub fn run(
 				type_name,
 				map_namespace,
 				templates::operation_response_common::OperationAction::Patch,
-				optional_feature,
+				operation_feature,
 			)?;
 
 			run_result.num_generated_structs += 1;
@@ -922,7 +945,7 @@ pub fn run(
 				type_name,
 				map_namespace,
 				templates::operation_response_common::OperationAction::Replace,
-				optional_feature,
+				operation_feature,
 			)?;
 
 			run_result.num_generated_structs += 1;
@@ -934,7 +957,7 @@ pub fn run(
 				type_name,
 				map_namespace,
 				templates::operation_response_common::OperationAction::Watch,
-				optional_feature,
+				operation_feature,
 			)?;
 
 			run_result.num_generated_structs += 1;
@@ -981,6 +1004,53 @@ pub fn run(
 		},
 	}
 
+	if let GenerateSchema::Yes { feature: schema_feature } = generate_schema {
+		match &definition.kind {
+			swagger20::SchemaKind::Properties(_) |
+			swagger20::SchemaKind::Ty(
+				swagger20::Type::Any |
+				swagger20::Type::Array { .. } |
+				swagger20::Type::Boolean |
+				swagger20::Type::Integer { .. } |
+				swagger20::Type::IntOrString |
+				swagger20::Type::Number { .. } |
+				swagger20::Type::Object { .. } |
+				swagger20::Type::String { .. } |
+				swagger20::Type::JsonSchemaPropsOrArray(_) |
+				swagger20::Type::JsonSchemaPropsOrBool(_) |
+				swagger20::Type::JsonSchemaPropsOrStringArray(_) |
+				swagger20::Type::Patch
+			) => {
+				templates::impl_schema::generate(
+					&mut out,
+					type_name,
+					Default::default(),
+					definition_path,
+					definition,
+					schema_feature,
+					map_namespace,
+				)?;
+			} 
+
+			swagger20::SchemaKind::Ty(swagger20::Type::WatchEvent(_)) => {
+				templates::impl_schema::generate(
+					&mut out,
+					type_name,
+					templates::Generics {
+						type_part: Some("T"),
+						where_part: None,
+					},
+					definition_path,
+					definition,
+					schema_feature,
+					map_namespace,
+				)?;
+			}
+
+			_ => (),
+		}
+	}
+
 	state.finish(out);
 
 	Ok(run_result)
@@ -1007,27 +1077,36 @@ fn get_derives(
 		return Ok(None);
 	}
 
-	let derive_clone = evaluate_trait_bound(kind, true, definitions, map_namespace, |kind, _| match kind {
-		swagger20::SchemaKind::Ty(swagger20::Type::CreateResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::DeleteResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ListResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::PatchResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ReplaceResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::WatchResponse) => Ok(false),
+	let derive_clone =
+		evaluate_trait_bound(
+			kind,
+			true,
+			definitions,
+			map_namespace,
+			|kind, _| Ok(!matches!(kind, swagger20::SchemaKind::Ty(
+				swagger20::Type::CreateResponse |
+				swagger20::Type::DeleteResponse |
+				swagger20::Type::ListResponse |
+				swagger20::Type::PatchResponse |
+				swagger20::Type::ReplaceResponse |
+				swagger20::Type::WatchResponse
+			))))?;
 
-		_ => Ok(true),
-	})?;
-
-	let derive_copy = derive_clone && evaluate_trait_bound(kind, false, definitions, map_namespace, |kind, _| match kind {
-		swagger20::SchemaKind::Ty(swagger20::Type::CreateOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::DeleteOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ListOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::PatchOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ReplaceOptional(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::WatchOptional(_)) => Ok(true),
-
-		_ => Ok(false),
-	})?;
+	let derive_copy =
+		derive_clone &&
+		evaluate_trait_bound(
+			kind,
+			false,
+			definitions,
+			map_namespace,
+			|kind, _| Ok(matches!(kind, swagger20::SchemaKind::Ty(
+				swagger20::Type::CreateOptional(_) |
+				swagger20::Type::DeleteOptional(_) |
+				swagger20::Type::ListOptional(_) |
+				swagger20::Type::PatchOptional(_) |
+				swagger20::Type::ReplaceOptional(_) |
+				swagger20::Type::WatchOptional(_)
+			))))?;
 
 	#[allow(clippy::match_same_arms)]
 	let is_default = evaluate_trait_bound(kind, false, definitions, map_namespace, |kind, required| match kind {
@@ -1049,17 +1128,19 @@ fn get_derives(
 		swagger20::SchemaKind::Ty(swagger20::Type::String { format: Some(swagger20::StringFormat::DateTime) }) => Ok(false),
 
 		// Enums without a default value
-		swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrArray(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrBool(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrStringArray(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::Patch) |
-		swagger20::SchemaKind::Ty(swagger20::Type::WatchEvent(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::CreateResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::DeleteResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ListResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::PatchResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ReplaceResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::WatchResponse) => Ok(false),
+		swagger20::SchemaKind::Ty(
+			swagger20::Type::JsonSchemaPropsOrArray(_) |
+			swagger20::Type::JsonSchemaPropsOrBool(_) |
+			swagger20::Type::JsonSchemaPropsOrStringArray(_) |
+			swagger20::Type::Patch |
+			swagger20::Type::WatchEvent(_) |
+			swagger20::Type::CreateResponse |
+			swagger20::Type::DeleteResponse |
+			swagger20::Type::ListResponse |
+			swagger20::Type::PatchResponse |
+			swagger20::Type::ReplaceResponse |
+			swagger20::Type::WatchResponse
+		) => Ok(false),
 
 		_ => Ok(true),
 	})?;
@@ -1068,16 +1149,20 @@ fn get_derives(
 		// IntOrString has a manual Default impl, so don't #[derive] it.
 		!matches!(kind, swagger20::SchemaKind::Ty(swagger20::Type::IntOrString));
 
-	let derive_partial_eq = evaluate_trait_bound(kind, true, definitions, map_namespace, |kind, _| match kind {
-		swagger20::SchemaKind::Ty(swagger20::Type::CreateResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::DeleteResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ListResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::PatchResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::ReplaceResponse) |
-		swagger20::SchemaKind::Ty(swagger20::Type::WatchResponse) => Ok(false),
-
-		_ => Ok(true),
-	})?;
+	let derive_partial_eq =
+		evaluate_trait_bound(
+			kind,
+			true,
+			definitions,
+			map_namespace,
+			|kind, _| Ok(!matches!(kind, swagger20::SchemaKind::Ty(
+				swagger20::Type::CreateResponse |
+				swagger20::Type::DeleteResponse |
+				swagger20::Type::ListResponse |
+				swagger20::Type::PatchResponse |
+				swagger20::Type::ReplaceResponse |
+				swagger20::Type::WatchResponse
+			))))?;
 
 	// The choice of deriving Eq, Ord and PartialOrd is deliberately more conservative than the choice of deriving PartialEq,
 	// so as to not change dramatically between Kubernetes versions. For example, ObjectMeta is Ord in v1.15 but not in v1.16 because
@@ -1087,10 +1172,10 @@ fn get_derives(
 
 	let derive_eq =
 		derive_partial_eq &&
-		matches!(kind,
-			swagger20::SchemaKind::Ty(swagger20::Type::IntOrString) |
-			swagger20::SchemaKind::Ty(swagger20::Type::String { format: Some(swagger20::StringFormat::DateTime) })
-		);
+		matches!(kind, swagger20::SchemaKind::Ty(
+			swagger20::Type::IntOrString |
+			swagger20::Type::String { format: Some(swagger20::StringFormat::DateTime) }
+		));
 
 	let derive_partial_ord =
 		derive_partial_eq &&
@@ -1155,7 +1240,7 @@ fn evaluate_trait_bound(
 			swagger20::SchemaKind::Ref(ref_path @ swagger20::RefPath { .. }) if !ref_path.references_scope(map_namespace) => {
 				#[allow(clippy::option_if_let_else)]
 				let trait_bound =
-					if let Some(target) = definitions.get(&swagger20::DefinitionPath(ref_path.path.to_owned())) {
+					if let Some(target) = definitions.get(&swagger20::DefinitionPath(ref_path.path.clone())) {
 						let mut visited = visited.clone();
 						evaluate_trait_bound_inner(
 							&std::borrow::Cow::Borrowed(&target.kind),
@@ -1168,7 +1253,7 @@ fn evaluate_trait_bound(
 						)
 					}
 					else {
-						f(&kind, required)
+						f(kind, required)
 					};
 				trait_bound
 			},
@@ -1184,9 +1269,11 @@ fn evaluate_trait_bound(
 					f,
 				),
 
-			swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrArray(namespace)) |
-			swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrBool(namespace)) |
-			swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrStringArray(namespace)) => {
+			swagger20::SchemaKind::Ty(
+				swagger20::Type::JsonSchemaPropsOrArray(namespace) |
+				swagger20::Type::JsonSchemaPropsOrBool(namespace) |
+				swagger20::Type::JsonSchemaPropsOrStringArray(namespace)
+			) => {
 				let json_schema_props_ref_path = swagger20::RefPath {
 					path: format!("io.k8s.apiextensions-apiserver.pkg.apis.apiextensions.{}.JSONSchemaProps", namespace),
 					can_be_default: None,
@@ -1205,7 +1292,7 @@ fn evaluate_trait_bound(
 					return Ok(false);
 				}
 
-				f(&kind, required)
+				f(kind, required)
 			},
 
 			swagger20::SchemaKind::Ty(swagger20::Type::WatchEvent(raw_extension_ref_path)) => {
@@ -1223,7 +1310,7 @@ fn evaluate_trait_bound(
 					return Ok(false);
 				}
 
-				f(&kind, required)
+				f(kind, required)
 			},
 
 			kind => f(kind, required),
@@ -1271,7 +1358,7 @@ fn get_fully_qualified_type_name(
 			result.push_str(&*get_rust_ident(namespace_part));
 			result.push_str("::");
 		}
-		result.push_str(&path_parts[path_parts.len() - 1]);
+		result.push_str(path_parts[path_parts.len() - 1]);
 		result
 	}
 	else {
@@ -1298,6 +1385,7 @@ pub fn get_rust_ident(name: &str) -> std::borrow::Cow<'static, str> {
 	// Eg "JSONSchemas" => "json_schemas", but "externalIPs" => "external_ips" instead of "external_i_ps".
 	// Mostly happens with plurals of abbreviations.
 	match name {
+		"clusterIPs" => return "cluster_ips".into(),
 		"externalIPs" => return "external_ips".into(),
 		"nonResourceURLs" => return "non_resource_urls".into(),
 		"podCIDRs" => return "pod_cidrs".into(),
@@ -1392,9 +1480,11 @@ fn get_rust_borrow_type(
 
 		swagger20::SchemaKind::Ty(swagger20::Type::IntOrString) => Err("nothing should be trying to refer to IntOrString".into()),
 
-		swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrArray(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrBool(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrStringArray(_)) => Err("JSON schema types not supported".into()),
+		swagger20::SchemaKind::Ty(
+			swagger20::Type::JsonSchemaPropsOrArray(_) |
+			swagger20::Type::JsonSchemaPropsOrBool(_) |
+			swagger20::Type::JsonSchemaPropsOrStringArray(_)
+		) => Err("JSON schema types not supported".into()),
 		swagger20::SchemaKind::Ty(swagger20::Type::Patch) => Err("Patch type not supported".into()),
 		swagger20::SchemaKind::Ty(swagger20::Type::WatchEvent(_)) => Err("WatchEvent type not supported".into()),
 
@@ -1468,9 +1558,11 @@ fn get_rust_type(
 
 		swagger20::SchemaKind::Ty(swagger20::Type::IntOrString) => Err("nothing should be trying to refer to IntOrString".into()),
 
-		swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrArray(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrBool(_)) |
-		swagger20::SchemaKind::Ty(swagger20::Type::JsonSchemaPropsOrStringArray(_)) => Err("JSON schema types not supported".into()),
+		swagger20::SchemaKind::Ty(
+			swagger20::Type::JsonSchemaPropsOrArray(_) |
+			swagger20::Type::JsonSchemaPropsOrBool(_) |
+			swagger20::Type::JsonSchemaPropsOrStringArray(_)
+		) => Err("JSON schema types not supported".into()),
 		swagger20::SchemaKind::Ty(swagger20::Type::Patch) => Err("Patch type not supported".into()),
 		swagger20::SchemaKind::Ty(swagger20::Type::WatchEvent(_)) => Err("WatchEvent type not supported".into()),
 
@@ -1508,7 +1600,7 @@ fn get_rust_type(
 ///
 /// - `type_name`: The name of the type that this operation is associated with, if any.
 ///
-/// - `optional_feature`: Specifies whether the API function will be emitted with a `#[cfg(feature = "<this value>")]` attribute or not.
+/// - `operation_feature`: Specifies whether the API function will be emitted with a `#[cfg(feature = "<this value>")]` attribute or not.
 ///
 /// # Returns
 ///
@@ -1522,7 +1614,7 @@ pub fn write_operation(
 	map_namespace: &impl MapNamespace,
 	vis: &str,
 	type_name: Option<&str>,
-	optional_feature: Option<&str>,
+	operation_feature: Option<&str>,
 ) -> Result<(Option<String>, Option<String>), Error> {
 	let local = map_namespace_local_to_string(map_namespace)?;
 
@@ -1710,8 +1802,8 @@ pub fn write_operation(
 		}
 	}
 
-	if let Some(optional_feature) = optional_feature {
-		writeln!(out, r#"{}#[cfg(feature = "{}")]"#, indent, optional_feature)?;
+	if let Some(operation_feature) = operation_feature {
+		writeln!(out, r#"{}#[cfg(feature = "{}")]"#, indent, operation_feature)?;
 	}
 
 	writeln!(out, "{}{}fn {}(", indent, vis, operation_fn_name)?;
@@ -1834,11 +1926,22 @@ pub fn write_operation(
 			for (parameter_name, parameter_type, parameter) in &parameters {
 				if parameter.location == swagger20::ParameterLocation::Query {
 					if parameter.required {
-						match parameter.schema.kind {
-							swagger20::SchemaKind::Ty(swagger20::Type::Boolean) |
-							swagger20::SchemaKind::Ty(swagger20::Type::Integer { .. }) |
-							swagger20::SchemaKind::Ty(swagger20::Type::Number { .. }) =>
-								writeln!(out, r#"{}    __query_pairs.append_pair({:?}, &{}.to_string());"#, indent, parameter.name, parameter_name)?,
+						match &parameter.schema.kind {
+							swagger20::SchemaKind::Ty(swagger20::Type::Array { items }) => match &items.kind {
+								swagger20::SchemaKind::Ty(swagger20::Type::String { .. }) => {
+									writeln!(out, "{}    for {} in {} {{", indent, parameter_name, parameter_name)?;
+									writeln!(out, r#"{}        __query_pairs.append_pair({:?}, {});"#, indent, parameter.name, parameter_name)?;
+									writeln!(out, "{}    }}", indent)?;
+								},
+
+								_ => return Err(format!("parameter {} is in the query string but is a {:?}", parameter.name, parameter_type).into()),
+							},
+
+							swagger20::SchemaKind::Ty(
+								swagger20::Type::Boolean |
+								swagger20::Type::Integer { .. } |
+								swagger20::Type::Number { .. }
+							) => writeln!(out, r#"{}    __query_pairs.append_pair({:?}, &{}.to_string());"#, indent, parameter.name, parameter_name)?,
 
 							swagger20::SchemaKind::Ty(swagger20::Type::String { .. }) =>
 								writeln!(out, r#"{}    __query_pairs.append_pair({:?}, &{});"#, indent, parameter.name, parameter_name)?,
@@ -1848,11 +1951,22 @@ pub fn write_operation(
 					}
 					else {
 						writeln!(out, "{}    if let Some({}) = {} {{", indent, parameter_name, parameter_name)?;
-						match parameter.schema.kind {
-							swagger20::SchemaKind::Ty(swagger20::Type::Boolean) |
-							swagger20::SchemaKind::Ty(swagger20::Type::Integer { .. }) |
-							swagger20::SchemaKind::Ty(swagger20::Type::Number { .. }) =>
-								writeln!(out, r#"{}        __query_pairs.append_pair({:?}, &{}.to_string());"#, indent, parameter.name, parameter_name)?,
+						match &parameter.schema.kind {
+							swagger20::SchemaKind::Ty(swagger20::Type::Array { items }) => match &items.kind {
+								swagger20::SchemaKind::Ty(swagger20::Type::String { .. }) => {
+									writeln!(out, "{}        for {} in {} {{", indent, parameter_name, parameter_name)?;
+									writeln!(out, r#"{}            __query_pairs.append_pair({:?}, {});"#, indent, parameter.name, parameter_name)?;
+									writeln!(out, "{}        }}", indent)?;
+								},
+
+								_ => return Err(format!("parameter {} is in the query string but is a {:?}", parameter.name, parameter_type).into()),
+							},
+
+							swagger20::SchemaKind::Ty(
+								swagger20::Type::Boolean |
+								swagger20::Type::Integer { .. } |
+								swagger20::Type::Number { .. }
+							) => writeln!(out, r#"{}        __query_pairs.append_pair({:?}, &{}.to_string());"#, indent, parameter.name, parameter_name)?,
 
 							swagger20::SchemaKind::Ty(swagger20::Type::String { .. }) =>
 								writeln!(out, r#"{}        __query_pairs.append_pair({:?}, {});"#, indent, parameter.name, parameter_name)?,
@@ -1962,8 +2076,8 @@ pub fn write_operation(
 			writeln!(out, "/// Optional parameters of [`{}`]", operation_fn_name)?;
 		}
 
-		if let Some(optional_feature) = optional_feature {
-			writeln!(out, r#"#[cfg(feature = "{}")]"#, optional_feature)?;
+		if let Some(operation_feature) = operation_feature {
+			writeln!(out, r#"#[cfg(feature = "{}")]"#, operation_feature)?;
 		}
 		writeln!(out, "#[derive(Clone, Copy, Debug, Default)]")?;
 		write!(out, "{}struct {}", vis, operation_optional_parameters_name)?;
@@ -2004,8 +2118,8 @@ pub fn write_operation(
 					operation_result_name, operation_fn_name)?;
 			}
 
-			if let Some(optional_feature) = optional_feature {
-				writeln!(out, r#"#[cfg(feature = "{}")]"#, optional_feature)?;
+			if let Some(operation_feature) = operation_feature {
+				writeln!(out, r#"#[cfg(feature = "{}")]"#, operation_feature)?;
 			}
 			writeln!(out, "#[derive(Debug)]")?;
 			writeln!(out, "{}enum {} {{", vis, operation_result_name)?;
@@ -2044,8 +2158,8 @@ pub fn write_operation(
 			writeln!(out, "}}")?;
 			writeln!(out)?;
 
-			if let Some(optional_feature) = optional_feature {
-				writeln!(out, r#"#[cfg(feature = "{}")]"#, optional_feature)?;
+			if let Some(operation_feature) = operation_feature {
+				writeln!(out, r#"#[cfg(feature = "{}")]"#, operation_feature)?;
 			}
 			writeln!(out, "impl {}Response for {} {{", local, operation_result_name)?;
 			writeln!(out, "    fn try_from_parts(status_code: {local}http::StatusCode, buf: &[u8]) -> Result<(Self, usize), {local}ResponseError> {{", local = local)?;
